--[[

$Revision: 15 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes", true)


----------------------------------------------------------------------
-- Constants

BOSS_NOTES_PLAYER_HEIGHT = 16
BOSS_NOTES_PLAYER_TEXT_WIDTH = 275
BOSS_NOTES_PLAYER_DISPLAYED_COUNT = 8
BOSS_NOTES_REFRESH_COOLDOWN = 10


---------------------------------------------------------------------
-- Subsystem

-- Initializes the Boss Notes frame subsystem
function BossNotes:InitializeBossNotesFrame ()
	-- Update
	self:UpdateRaids()
end

-- Enable the Boss Notes frame subsystem
function BossNotes:EnableBossNotesFrame ()
	self:RegisterEvent("UNIT_PORTRAIT_UPDATE")
	self:RegisterMessage("BOSS_NOTES_UPDATE_PLAYERS")
	self:RegisterMessage("BOSS_NOTES_ACTIVATION")
end

-- Disables the Boss Notes frame subsystem
function BossNotes:DisableBossNotesFrame ()
end


----------------------------------------------------------------------
-- Event handlers

-- OnLoad handler
function BossNotes:OnLoadBossNotesFrame (frame)
	-- Make the Boss Notes frame pushable
	BossNotesFrame:SetAttribute("UIPanelLayout-defined", true)
	BossNotesFrame:SetAttribute("UIPanelLayout-enabled", true)
	BossNotesFrame:SetAttribute("UIPanelLayout-area", "left")
	BossNotesFrame:SetAttribute("UIPanelLayout-pushable", 5)
	BossNotesFrame:SetAttribute("UIPanelLayout-whileDead", true)

	-- Translations
	BossNotesFrameTitleText:SetText(L["BOSS_NOTES_FRAME_TITLE"])
	BossNotesRefreshButton:SetText("Refresh")
end

-- OnShow handler
function BossNotes:OnShowBossNotesFrame (frame)
	PlaySound("igCharacterInfoOpen")
	SetPortraitTexture(BossNotesFramePortrait, "player")
	self:UpdatePlayers()
end

-- OnHide handler
function BossNotes:OnHideBossNotesFrame (frame)
	PlaySound("igCharacterInfoClose")
	self:ApplyPlayer()
end

-- Handles the clicking of the refresh button
function BossNotes:OnRefreshClick (frame)
	-- Force a player request on the network
	self:RequestPlayers(self.selectedKey, true)
	
	-- Disable the button and set a timer
	frame:Disable()
	self:ScheduleTimer("EnableRefresh", BOSS_NOTES_REFRESH_COOLDOWN)
end

-- Enables the refresh button
function BossNotes:EnableRefresh ()
	BossNotesRefreshButton:Enable()
end

-- Handles the UNIT_PORTRAIT_UPDATE event
function BossNotes:UNIT_PORTRAIT_UPDATE (message, unit)
	if unit == "player" then
		SetPortraitTexture(BossNotesFramePortrait, "player")
	end
end

-- Handles the BOSS_NOTES_UPDATE_PLAYERS message from the data subsystem
function BossNotes:BOSS_NOTES_UPDATE_PLAYERS (message, keys)
	if keys[self.selectedKey] then
		self:UpdatePlayers()
	end
end

function BossNotes:BOSS_NOTES_ACTIVATION (message, raid, encounter)
	-- Update raid
	UIDropDownMenu_SetSelectedValue(BossNotesRaidDropDown, raid)
	UIDropDownMenu_SetText(BossNotesRaidDropDown, raid.name)
	self.selectedRaid = raid
	self:UpdateEncounters(true)
	
	-- Update encounter
	UIDropDownMenu_SetSelectedValue(BossNotesEncounterDropDown, encounter)
	self.selectedEncounter = encounter
	self:UpdatePlayers()
end


----------------------------------------------------------------------
-- APIs

-- Returns the current selection
function BossNotes:GetSelection ()
	local selectedRaid = self.selectedRaid
	local selectedEncounter = self.selectedEncounter
	local selectedPlayer = self.selectedPlayer
	return selectedRaid, selectedEncounter, selectedPlayer
end


----------------------------------------------------------------------
-- Content and selection managment

-- Updates the raids drop down
function BossNotes:UpdateRaids (limitted)
	UIDropDownMenu_Initialize(BossNotesRaidDropDown, BossNotesRaidDropDown_Initialize)
	UIDropDownMenu_SetSelectedID(BossNotesRaidDropDown, 1)
	self.selectedRaid = nil
	if not limitted then
		self:UpdateEncounters()
	end
end

-- Initializes the raids drop down
function BossNotesRaidDropDown_Initialize ()
	-- Add "All Raids"
	local info = UIDropDownMenu_CreateInfo()
	info.text = "All Raids"
	info.func = BossNotes_OnRaidDropDownClick
	UIDropDownMenu_AddButton(info)

	-- Add effective raids
	local raids = BossNotes:GetRaids()
	for _, raid in ipairs(raids) do 
		info = UIDropDownMenu_CreateInfo()
		info.text = raid.name
		info.func = BossNotes_OnRaidDropDownClick
		info.value = raid
		UIDropDownMenu_AddButton(info)
	end
end

-- Handles the selection of a raid
function BossNotes_OnRaidDropDownClick (button)
	UIDropDownMenu_SetSelectedID(BossNotesRaidDropDown, button:GetID())
	BossNotes.selectedRaid = button.value
	if type(BossNotes.selectedRaid) ~= "table" then
		BossNotes.selectedRaid = nil
	end
	BossNotes:UpdateEncounters()
end

-- Updates the encounters drop down
function BossNotes:UpdateEncounters (limitted)
	UIDropDownMenu_Initialize(BossNotesEncounterDropDown, BossNotes_InitializeEncounterDropDown)
	UIDropDownMenu_SetSelectedID(BossNotesEncounterDropDown, 1)
	self.selectedEncounter = nil
	if not limitted then
		self:UpdatePlayers()
	end
end

-- Initializes the encounters drop down
function BossNotes_InitializeEncounterDropDown ()
	-- Add "All Encounters"
	local info = UIDropDownMenu_CreateInfo()
	info.text = "All Encounters"
	info.func = BossNotes_OnEncounterDropDownClick
	UIDropDownMenu_AddButton(info)
	
	-- Add effective encounters
	if BossNotes.selectedRaid then
		for _, encounter in ipairs(BossNotes.selectedRaid.encounters) do
			info = UIDropDownMenu_CreateInfo()
			info.text = encounter.name
			info.func = BossNotes_OnEncounterDropDownClick
			info.value = encounter
			UIDropDownMenu_AddButton(info)
		end
	end
end

-- Handles the selection of an encounter
function BossNotes_OnEncounterDropDownClick (button)
	UIDropDownMenu_SetSelectedID(BossNotesEncounterDropDown, button:GetID())
	BossNotes.selectedEncounter = button.value
	if type(BossNotes.selectedEncounter) ~= "table" then
		BossNotes.selectedEncounter = nil
	end
	BossNotes:UpdatePlayers()
end

-- Update the list of players
function BossNotes:UpdatePlayers (limitted)
	-- Update key
	local oldSelectedKey = self.selectedKey
	self.selectedKey = self:GetKey(self.selectedRaid, self.selectedEncounter)

	-- Request players (no force)
	self:RequestPlayers(self.selectedKey, false)
	
	-- Maintain selection
	local players = self:GetPlayers(self.selectedKey)
	local oldSelectedPlayer = self.selectedPlayer
	self.selectedPlayer = nil
	if oldSelectedPlayer then
		for _, player in ipairs(players) do
			if player.name == oldSelectedPlayer.name then
				self.selectedPlayer = player
				break
			end
		end
	end
	if not self.selectedPlayer then
		self.selectedPlayer = players[1]
	end

	-- Send selection change message
	if self.selectedKey ~= oldSelectedKey then
		self:SendMessage("BOSS_NOTES_SELECTION", self.selectedRaid, self.selectedEncounter)
	end
	
	-- Update player
	if not limitted then
		self:UpdatePlayerList()
	end
end

-- Update the player list
function BossNotes:UpdatePlayerList (limitted)
	-- Update the scroll framne
	local players = self:GetPlayers(self.selectedKey)
	local numPlayers = #players
	local playerOffset = FauxScrollFrame_GetOffset(BossNotesPlayerListScrollFrame)
	FauxScrollFrame_Update(BossNotesPlayerListScrollFrame, numPlayers, BOSS_NOTES_PLAYER_DISPLAYED_COUNT,
			BOSS_NOTES_PLAYER_HEIGHT, nil, nil, nil, BossNotesPlayerHighlightFrame, 293, 316)
	BossNotesPlayerHighlightFrame:Hide();
	for i = 1, BOSS_NOTES_PLAYER_DISPLAYED_COUNT do
		local playerButton = _G["BossNotesPlayer" .. i]
		local playerButtonText = _G["BossNotesPlayer" .. i .. "Text"]
		local playerIndex = i + playerOffset;
		local player = players[playerIndex]
		if playerIndex <= numPlayers then	
			if BossNotesPlayerListScrollFrame:IsShown() then
				playerButton:SetWidth(293)
			else
				playerButton:SetWidth(323);
			end
			local color = RAID_CLASS_COLORS[player.class]
			playerButtonText:SetTextColor(color.r, color.g, color.b)
			playerButton:SetText(" " .. player.name)
			playerButton:SetID(playerIndex)
			playerButton:Show()
			if player == self.selectedPlayer then
				BossNotesPlayerHighlightFrame:SetPoint("TOPLEFT", "BossNotesPlayer" .. i, "TOPLEFT", 2, 0)
				BossNotesPlayerHighlightFrame:Show()
				playerButton:LockHighlight()
				playerButton.isHighlighted = true
			else
				playerButton:UnlockHighlight()
				playerButton.isHighlighted = false
			end
		else
			playerButton:Hide();
		end
	end
	
	-- Update the player
	if not limitted then
		self:UpdatePlayer()
	end
end

-- Regular function invoked by the scroll bar
function BossNotesFrame_UpdatePlayerList ()
	BossNotes:UpdatePlayerList()
end

-- Handles the clicking of a player button
function BossNotes:OnPlayerButtonClick (frame, button)
	if button == "LeftButton" then
		local players = self:GetPlayers(self.selectedKey)
		self.selectedPlayer = players[frame:GetID()]
		self:UpdatePlayerList()
	end
end

-- Update the player
function BossNotes:UpdatePlayer (limitted)
	-- Apply pending changes
	self:ApplyPlayer()
	
	-- Update
	local player = self.selectedPlayer
	if player.note then
		BossNotesNote:SetText(player.note)
		BossNotesNote:SetCursorPosition(0)
		BossNotesMetaData:SetText(string.format("%s (%s)",
				date("%Y-%m-%d %H:%M", player.time),
				player.clientVersion))
	else
		BossNotesNote:SetText("")
		BossNotesMetaData:SetText("")
	end
	self.canEditNote = player.name == self.playerName
	self.noteKey = self.selectedKey
	
	-- Enable/Disable
	if self.canEditNote then
		BossNotesNote:EnableMouse(true)
		BossNotesNote:SetTextColor(1.0, 1.0, 1.0)
	else
		BossNotesNote:EnableMouse(false)
		BossNotesNote:SetTextColor(0.7, 0.7, 0.7)
		BossNotesNote:ClearFocus()
	end
end

-- Applies the changes made by the player
function BossNotes:ApplyPlayer ()
	if self.canEditNote then
		self:SetPlayerNote(self.noteKey, BossNotesNote:GetText())
	end
end
