--[[

$Revision: 16 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes", true)


----------------------------------------------------------------------
-- Constants

-- Cache TTL
BOSS_NOTES_TTL = 300

-- Raids and encounters
BOSS_NOTES_RAIDS = {
	-- Classic
	{
	},
	
	-- The Burning Crusade
	{
	},
	
	-- Wrath of the Lich King
	{
		{
			raid = "vault-of-archavon",
			name = "Vault of Archavon",
			encounters = {
				{
					encounter = 31125,
					name = "Archavon the Stone Watcher"
				}
			}
		},
		{
			raid = "naxxramas",
			name = "Naxxramas",
			encounters = {
				{
					encounter = 15956,
					name = "Anub'rekhan"
				},
				{
					encounter = 15953,
					name = "Grand Widow Faerlina"
				},
				{
					encounter = 15952,
					name = "Maexxna"
				},
				{
					encounter = 15954,
					name = "Noth the Plaguebringer"
				},
				{
					encounter = 15936,
					name = "Heigan the Unclean"
				},
				{
					encounter = 16011,
					name = "Loatheb"
				},
				{
					encounter = 16061,
					name = "Instructor Razuvious"
				},
				{
					encounter = 16060,
					name = "Gothik the Harvester"
				},
				{
					encounter = 16063,
					name = "The Four Horsemen",
					npcs = { 16063, 16064, 16065, 30549 }
				},
				{
					encounter = 16028,
					name = "Patchwerk"
				},
				{
					encounter = 15931,
					name = "Grobbulus"
				},
				{
					encounter = 15932,
					name = "Gluth"
				},
				{
					encounter = 15928,
					name = "Thaddius"
				},
				{
					encounter = 15989,
					name = "Sapphiron"
				},
				{
					encounter = 15990,
					name = "Kel'Thuzad"
				}
			}
		},
		{
			raid = "the-obsidian-sanctum",
			name = "The Obsidian Sanctum",
			encounters = {
				{
					encounter = 28860,
					name = "Sartharion",
					npcs = { 28860, 30449, 30451, 30452 }
				}
			}
		},
		{
			raid = "the-eye-of-eternity",
			name = "The Eye of Eternity",
			encounters = {
				{
					encounter = 29655,
					name = "Malygos"
				}
			}
		}
	}
}
	

---------------------------------------------------------------------
-- Subsystem

-- Initializes the data subsystem
function BossNotes:InitializeData ()
	-- Player name and class
	self.playerName = UnitName("player")
	_, self.playerClass = UnitClass("player")
	
	-- Cache
	self.cache = { }
	
	-- Players
	self.players = { }
	
	-- NPC to raid/encounter map
	self.raidEncounters = { }
	for _, gameVersion in ipairs(BOSS_NOTES_RAIDS) do
		for _, raid in ipairs(gameVersion) do
			for _, encounter in ipairs(raid.encounters) do
				local raidEncounter = {
					raid = raid,
					encounter = encounter
				}
				self.raidEncounters[encounter.encounter] = raidEncounter
				if encounter.npcs then
					for _, npc in ipairs (encounter.npcs) do
						self.raidEncounters[npc] = raidEncounter
					end
				end
			end
		end
	end
end

-- Enables the data subsystem
function BossNotes:EnableData ()
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterMessage("BOSS_NOTES_REQUEST")
	self:RegisterMessage("BOSS_NOTES_RESPONSE")
end

-- Disables the data subsystem
function BossNotes:DisableData  ()
end

----------------------------------------------------------------------
-- Event handlers

-- Handles the UPDATE_MOUSEOVER_UNIT event
function BossNotes:UPDATE_MOUSEOVER_UNIT ()
	self:CheckUnit("mouseover")
end

-- Handles the PLAYER_TARGET_CHANGED event
function BossNotes:PLAYER_TARGET_CHANGED ()
	self:CheckUnit("target")
end

-- Handles a request
function BossNotes:BOSS_NOTES_REQUEST (message, key, distribution)
	-- Note time of request
	local keyCache = self:GetKeyCache(key)
	keyCache.lastRequest = time()
	
	-- Send response
	local note = self.db.char.notes[key]
	if note then
		self:SendResponse(key, note.note, note.time, note.clientVersion, distribution)
	end
end

-- Handles a response
function BossNotes:BOSS_NOTES_RESPONSE (message, key, name, class, note, time, clientVersion)
	-- Ignore our own response
	if name == self.playerName then
		return
	end
	
	-- Cache response
	local keyCache = self:GetKeyCache(key)
	if not keyCache.players[name] then
		keyCache.players[name] = { }
		
		-- Invalidate players
		self.players[key] = nil
	end
	local player = keyCache.players[name]
	player.name = name
	player.class = class
	player.note = note
	player.time = time
	player.clientVersion = clientVersion
	
	-- Schedule update
	if not self.updatePlayersTimer then
		self.updatePlayersTimer = self:ScheduleTimer("SendUpdatePlayers", 1.0)
	end
	if not self.updatePlayersKeys then
		self.updatePlayersKeys = { }
	end
	self.updatePlayersKeys[key] = true
end

-- Sends the players update message once the timer has expired
function BossNotes:SendUpdatePlayers ()
	-- Clean
	self.updatePlayersTimer = nil
	local keys = self.updatePlayersKeys
	self.updatePlayersKeys = nil
	
	-- Send message
	self:SendMessage("BOSS_NOTES_UPDATE_PLAYERS", keys)
end


----------------------------------------------------------------------
-- APIs

-- Returns the raids
function BossNotes:GetRaids ()
	local raids = { }
	if self.db.global.gameVersionClassic then
		for _, raid in ipairs(BOSS_NOTES_RAIDS[1]) do
			table.insert(raids, raid)
		end
	end
	if self.db.global.gameVersionTbc then
		for _, raid in ipairs(BOSS_NOTES_RAIDS[2]) do
			table.insert(raids, raid)
		end
	end
	if self.db.global.gameVersionWotlk then
		for _, raid in ipairs(BOSS_NOTES_RAIDS[3]) do
			table.insert(raids, raid)
		end
	end
	return raids
end

-- Requests the players on the network
function BossNotes:RequestPlayers (key, force)
	-- Request from network if TTL has expired
	local time = time()
	local keyCache = self:GetKeyCache(key)
	if time - keyCache.lastRequest > BOSS_NOTES_TTL or force then
		self:SendRequest(key)
	end
end

-- Returns the player notes
function BossNotes:GetPlayers (key)
	-- Update players as needed
	if not self.players[key] then
		self.players[key] = { }
		local players = self.players[key]
	
		-- Add player
		local player = {
			name = self.playerName,
			class = self.playerClass
		}
		if self.db.char.notes[key] then
			player.note = self.db.char.notes[key].note
			player.time = self.db.char.notes[key].time
			player.clientVersion = self.db.char.notes[key].clientVersion
		end
		table.insert(players, player)

		-- Add from cache
		local keyCache = self:GetKeyCache(key)
		for _, player in pairs(keyCache.players) do
			table.insert(players, player)
		end
		
		-- Sort
		table.sort(players, function (a, b)
			-- Player goes first
			if a.name == self.playerName then
				return true
			end
			if b.name == self.playerName then
				return false
			end
			
			-- Most recently updated notes go first
			if a.time > b.time then
				return true
			end
		end)
	end
	
	return self.players[key]
end

-- Sets the note of the player
function BossNotes:SetPlayerNote (key, note)
	-- Normalize note to nil if empty
	if note then
		note = strtrim(note)
		if string.len(note) == 0 then
			note = nil
		end
	end
	
	-- Set or clear, writing through to the players structure
	local players = self.players[key]
	if note then
		-- Detect changes
		if not self.db.char.notes[key] or note ~= self.db.char.notes[key].note then
			local time = time()
			local clientVersion = GetBuildInfo()
			self.db.char.notes[key] = {
				note = note,
				time = time,
				clientVersion = clientVersion
			}
			if players then
				players[1].note = note
				players[1].time = time
				players[1].clientVersion = clientVersion
			end
		end
	else
		self.db.char.notes[key] = nil
		if players then
			players[1].note = nil
			players[1].time = nil
			players[1].clientVersion = nil
		end
	end
end

-- Returns a key
function BossNotes:GetKey (raid, encounter) 
	if raid then
		if encounter then
			return "encounter:" .. tostring(self.selectedEncounter.encounter)
		else
			return "raid:" .. self.selectedRaid.raid
		end
	else
		return "general"
	end
end


----------------------------------------------------------------------
-- Support methods

-- Returns the cache for a key
function BossNotes:GetKeyCache (key)
	if not self.cache[key] then
		self.cache[key] = {
			lastRequest = 0,
			players = { }
		}
	end
	return self.cache[key]
end

-- Checks a unit
function BossNotes:CheckUnit (unit)
	-- Get the GUID
	local guid = UnitGUID(unit)
	
	-- Test whether the unit is a creature
	if guid and string.sub(guid, 5, 5) == "3" then
		-- Get the NPC ID of the unit
		local npc = tonumber("0x" .. string.sub(guid, 9, 12))
		
		-- Find and process a mapped raid encounter
		local raidEncounter = self.raidEncounters[npc]
		if raidEncounter then
			self:SendMessage("BOSS_NOTES_ACTIVATION", raidEncounter.raid, raidEncounter.encounter)
		end
	end
end
